# -*- coding: utf-8 -*-
#
# license under WTFPL (Do What the Fuck You Want)
# author woo

require 'digest/md5'
require 'net/http'
require 'uri'

class UpYun
  attr_accessor :bucket, :username, :password
  attr_reader :endpoint

  def initialize(bucket, username, password, endpoint=0)
    @bucket = bucket
    @username = username
    @password = password
    self.endpoint = endpoint
  end

  # public API

  def endpoint=(value)
    @endpoint = "v#{value}.api.upyun.com"
  end

  # 上传文件
  def put(path, data, auto_mkdir=true, checksum=nil, options={})
    headers = {}
    headers['mkdir'] = auto_mkdir.to_s
    headers['Content-MD5'] = checksum if checksum
    headers['Content-Length'] = data.size.to_s
    headers.merge(options)
    res = http_request('PUT', path, data, headers)
    res_msg(res)
  end

  # 下载文件
  def get(path)
    res = http_request('GET', path)
    res_msg(res, res.body)
  end

  # 获取文件信息
  def getinfo(path)
    res = http_request('HEAD', path)
    if res.code.eql? '200'
      {
        :file_type => res['x-upyun-file-type'],
        :file_size => res['x-upyun-file-size'],
        :file_date => res['x-upyun-file-date']
      }
    else
      errmsg(res)
    end
  end

  # 创建目录
  def mkdir(path, auto_mkdir=true)
    headers = {'folder' => 'true'}
    headers['mkdir'] = auto_mkdir.to_s
    res = http_request('POST', path, nil, headers)
    res_msg(res)
  end

  # 删除目录或文件
  def delete(path)
    res = http_request('DELETE', path)
    res_msg(res)
  end

  # 获取目录文件列表
  def getlist(path='/')
    res = http_request('GET', path)
    if res.code.eql? '200'
      res.body.split("\n").map do |file|
        Hash[[:name, :type, :size, :time].zip(file.split("\t"))]
      end
    else
      errmsg(res)
    end
  end

  # 获取空间使用情况
  def usage(path='/')
    res = http_request('GET', "#{path}?usage")
    res_msg(res, res.body)
  end

  private

  def res_msg(res, msg=true)
    (res.code.eql? '200') ? msg : errmsg(res)
  end

  def errmsg(res)
    { :code => res.code, :message => res.message }
  end

  def http_request(method, path, data=nil, headers={})
    url = "http://#{@endpoint}/#{@bucket}#{path}"
    uri = URI.parse(URI.encode(url))

    date = Time.now.utc.strftime('%a, %d %b %Y %H:%M:%S GMT')
    length = headers['Content-Length'] || 0
    headers['Date'] = date
    headers['Authorization'] = make_signature(method, uri.path, date, length)
    Net::HTTP.start(uri.host, uri.port) do |http|
      if method == 'HEAD'
        return http.head(uri.request_uri, headers)
      else
        return http.send_request(method, uri.request_uri, data, headers)
      end
    end
  end

  def make_signature(method, uri, date, length)
    signature = "#{method}&#{uri}&#{date}&#{length}&#{md5(@password)}"
    "UpYun #{@username}:#{md5(signature)}"
  end

  def md5(data)
    Digest::MD5.hexdigest(data)
  end
end
